import cv2 as cv
import numpy as np
import tkinter as tk
from tkinter import filedialog, messagebox
import os
from PIL import Image, ImageTk
import configparser
from ImageIO import ImageIO

##
# @brief
# A class to handle various commands related to image processing and display.
# The Commands class provides methods for loading, displaying, and adjusting the overlap region for images within a graphical user interface (GUI).
# It also supports image blending for overlapping regions with smooth gradient transitions.
class Commands:
    ##
    # @brief
    # Initializes the Commands instance.
    # Sets up the GUI interface, shared data, and ImageIO for image operations.
    #
    # @param gui The GUI instance used for displaying images and receiving user input.
    # @param shared_data The shared data structure that holds images and other state information.
    # @return None
    def __init__(self, gui, shared_data):
        self.gui = gui
        self.shared_data = shared_data
        self.io = ImageIO(shared_data)

    ##
    # @brief Loads the left image from a file.
    # Opens a file dialog for the user to select a left image file. If a file is selected, it is loaded into the shared data.
    # @return None
    #
    # @warning This method does not check the file format or image content.
    def load_left_image(self):
        filepath = filedialog.askopenfilename(title="Select Left Image")
        if filepath:
            self.shared_data.left_image = self.io.load_image(filepath)
            print("Left image loaded.")

    ##
    # @brief Loads the right image from a file.
    # Opens a file dialog for the user to select a right image file. If a file is selected, it is loaded into the shared data.
    #
    # @return None
    # @warning This method does not check the file format or image content.
    def load_right_image(self):
        filepath = filedialog.askopenfilename(title="Select Right Image")
        if filepath:
            self.shared_data.right_image = self.io.load_image(filepath)
            print("Right image loaded.")

    ## 
    # @brief Creates a smooth gradient for the overlap region.
    # Generates a gradient that transitions from full brightness to zero using a non-linear quadratic falloff.
    # The gradient is scaled by the specified rate and clipped to be between 0 and 1.
    #
    # @param length The length of the gradient.
    # @param rate A scaling factor to control the gradient's intensity.
    # @return A numpy array containing the gradient.
    def smooth_gradient(self,length, rate):
        # Create a linear gradient that transitions from full brightness (1) to zero (0)
        x = np.linspace(0, 1, length)
        # Apply a non-linear function (e.g., quadratic) for a smoother transition
        gradient = 1 - (x ** 2)  # Quadratic falloff to create a smooth transition
        gradient = np.clip(gradient * rate, 0, 1)  # Scale the gradient by the rate
        return gradient

    ##
    # @brief
    # Displays the left image with a smooth gradient applied to the overlap region.
    # If a left image is available, this method applies a smooth gradient to its overlapping region and displays it on the GUI canvas.
    # If no left image is loaded, an error message is shown.
    #
    # @return None
    def show_left_image(self):
        if self.shared_data.left_image is not None:
            self.shared_data.isLeft = True
            left = self.shared_data.left_image.copy()
            height = left.shape[0]
            left_overlap_start = left.shape[1] - self.shared_data.overlap_region
            
            # Create a smooth gradient alpha mask for the left image (1 to 0)
            gradient_alpha = np.linspace(1, 0, self.shared_data.overlap_region)
            gradient_alpha = np.tile(gradient_alpha, (height, 1))

            # Apply the alpha gradient to the overlapping region
            left_overlap = left[:, left_overlap_start:, :].copy()
            left_overlap = (left_overlap * gradient_alpha[:, :, np.newaxis]).astype(np.uint8)

            # Replace the modified overlapping region back into the original left image
            left[:, left_overlap_start:, :] = left_overlap

            # Show the modified image
            self.io.show_image(left, canvas=self.gui.canvas)
            print("Left image displayed.")
        else:
            messagebox.showerror("Error", "No left image loaded!")

    ## 
    # @brief 
    # Displays the right image with a smooth gradient applied to the overlap region.
    # If a right image is available, this method applies a smooth gradient to its overlapping region and displays it on the GUI canvas.
    # If no right image is loaded, an error message is shown.
    #
    # @return None
    def show_right_image(self):
        if self.shared_data.right_image is not None:
            self.shared_data.isLeft = False
            right = self.shared_data.right_image.copy()
            height = right.shape[0]
            right_overlap_end = self.shared_data.overlap_region
            
            # Create a smooth gradient alpha mask for the right image (0 to 1)
            gradient_alpha = np.linspace(0, 1, right_overlap_end)
            gradient_alpha = np.tile(gradient_alpha, (height, 1))

            # Apply the alpha gradient to the overlapping region
            right_overlap = right[:, :right_overlap_end, :].copy()
            right_overlap = (right_overlap * gradient_alpha[:, :, np.newaxis]).astype(np.uint8)

            # Replace the modified overlapping region back into the original right image
            right[:, :right_overlap_end, :] = right_overlap

            # Show the modified image
            self.io.show_image(right, canvas=self.gui.canvas)
            print("Right image displayed.")
        else:
            messagebox.showerror("Error", "No right image loaded!")
    
    ##
    # @brief
    # Displays the original left image without any modifications.
    # If a left image is loaded, it displays the original left image on the GUI canvas.
    #
    # @return None
    def show_right_original(self):

        # Show original image
        if self.shared_data.right_image is not None:
            self.shared_data.isLeft = False
            right = self.shared_data.right_image.copy()
            self.io.show_image(right, canvas=self.gui.canvas)
            print("Right image displayed.")
        else:
            messagebox.showerror("Error", "No right image loaded!")

    ##
    # @brief Displays the original right image without any modifications.
    # If a right image is loaded, it displays the original right image on the GUI canvas.
    #
    # @return Nothing returned.
    def show_left_original(self):

        # Show original image
        if self.shared_data.left_image is not None:
            self.shared_data.isLeft = True
            left = self.shared_data.left_image.copy()
            self.io.show_image(left, canvas=self.gui.canvas)
            print("Left image displayed.")
        else:
            messagebox.showerror("Error", "No left image loaded!")

    ## 
    # @brief Sets the overlap region for image processing.
    # Updates the overlap region value from the GUI input, updates the shared data, and redisplays the current image (left or right) with the new overlap setting.
    #
    # @return None
    def set_overlap_region(self):
        self.shared_data.overlap_region = self.gui.overlap.get()
        print(f"Overlap region set to {self.shared_data.overlap_region}")
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()
    
    ## 
    # @brief Increases the overlap region value by one.
    # Increments the overlap region by one and redisplays the current image with the updated overlap.
    #
    # @return None
    def increase_overlap(self):
        current_overlap = self.gui.overlap.get()
        self.gui.overlap.set(current_overlap + 1)
        self.set_overlap_region()
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()
    
    ## 
    # @brief
    # Decreases the overlap region value by one.
    # Decrements the overlap region by one (ensuring it does not go negative) and redisplays the current image with the updated overlap.
    #
    # @return None
    def decrease_overlap(self):
        current_overlap = self.gui.overlap.get()
        self.gui.overlap.set(max(0, current_overlap - 1))  # Prevents negative overlap
        self.set_overlap_region()
        if(self.shared_data.isLeft):
            self.show_left_image()
        else:
            self.show_right_image()

