import cv2 as cv
import numpy as np
import tkinter as tk
from tkinter import filedialog, messagebox
import os
from PIL import Image, ImageTk
import configparser

## 
# @brief
# This class is used to store shared data between different classes.
# The SharedData class maintains information such as images, overlap region width, and a flag for which side (left or right) is active.
# This data is accessible to other classes, allowing them to read and modify these shared attributes.
# The class also includes methods for saving and loading the settings to and from a configuration file.
class SharedData:
    ## @brief
    # Initializes the SharedData instance.
    # The constructor sets up initial values for shared data attributes.
    def __init__(self):
        self.left_image = None
        self.right_image = None
        self.overlap_region = 200
        self.leftSide = True

    ##
    # @brief Saves current settings to a configuration file.
    # This method writes the current values of `overlap_region` and `leftSide` to a configuration file 
    # (default name: `config.ini`) in the [DEFAULT] section. This allows the settings to be persisted
    # across sessions and loaded later.
    #
    # @param filename The name of the configuration file to write to, defaulting to 'config.ini'.
    # @param self the object pointer
    # @return None
    def write_to_config(self, filename='config.ini'):
        config = configparser.ConfigParser()
        config['DEFAULT'] = {
            'overlap_region': str(self.overlap_region),
            'leftSide': str(self.leftSide)
        }
        with open(filename, 'w') as configfile:
            config.write(configfile)

    ##
    # @brief Loads settings from a configuration file.
    # This method reads `overlap_region` and `leftSide` values from a specified configuration file 
    # (default name: `config.ini`). If the file is not found, default values are retained.
    #
    # @param filename The name of the configuration file to read from, defaulting to 'config.ini'.
    # @return None
    def read_from_config(self, filename='config.ini'):
        config = configparser.ConfigParser()
        config.read(filename)
        self.overlap_region = config['DEFAULT'].getint('overlap_region', self.overlap_region)
        self.leftSide = config['DEFAULT'].getboolean('leftSide', self.leftSide)
